#! /usr/bin/guile \
--debug -e main -s
!#

;;;;
;;;; gschem-to-sexp -- Convert a schematic or symbol file into an
;;;; s-expression.
;;;;
;;;; Copyright 2007 Dean Ferreyra, All rights reserved
;;;;
;;;; This file is part of octw-guile-geda.
;;;;
;;;; octw-guile-geda is free software: you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; octw-guile-geda is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with octw-guile-geda.  If not, see <http://www.gnu.org/licenses/>.
;;;;
;;;; $Id: gschem-to-sexp.scm,v 1.6 2008-09-04 06:05:17 dean Exp $
;;;; Dean Ferreyra

(use-modules (srfi srfi-1)
             (srfi srfi-13)
             (srfi srfi-19))
(use-modules (ice-9 optargs)
             (ice-9 regex)
             (ice-9 getopt-long)
             (ice-9 pretty-print))
(use-modules (octw conditional)
             (octw anaphoric)
             (octw file-system)
             (octw gaf)
             (octw gaf-v1)
             (octw gaf-read))

(use-syntax (ice-9 syncase))

;;; Generate gEDA/gaf version 1 objects plus reader
(gaf-gen-v1-specs)
(gaf-read-gen)

(define *cmd* "gschem-to-sexp.scm")

(define (usage)
  (format #t "usage: ~A [OPTIONS]... [<gschem-file>]~%" *cmd*)
  (format #t "  -o, --output        output file name~%")
  (format #t "  -p, --pretty-print  pretty-print the output~%")
  (format #t "  -w, --width=WIDTH   columns for pretty-print~%")
  (format #t "  -h, --help          this usage message~%")
  (exit 0))

(define (announce-error template . args)
  (format #t "~A: " *cmd*)
  (apply format #t template args)
  (format #t "~%")
  (exit 1))

(define option-spec
  `((output (single-char #\o) (value #t))
    (pretty-print (single-char #\p))
    (width (single-char #\w) (value #t))
    (help (single-char #\h))))

(define (main args)
  (let ((options (getopt-long (command-line) option-spec)))
    (when (option-ref options 'help #f)
      (usage))
    (let ((files (option-ref options '() '())))
      (unless (<= (length files) 1)
        (display-error "Zero or one files expected"))
      (call-with-input-file-or-port
          (and (not (null? files))
               (first files))
          (current-input-port)
        (lambda (in)
          (call-with-output-file-or-port
              (option-ref options 'output #f)
              (current-output-port)
            (lambda (out)
              (let ((sexp (gaf-read in)))
                (if (option-ref options 'pretty-print #f)
                    (pretty-print sexp out
                                  #:width (option-ref options 'width 79))
                    (write sexp out))
                (newline out)))))))))
